@@ -1,5 +1,5 @@
 /*
- * iperf, Copyright (c) 2014, The Regents of the University of
+ * iperf, Copyright (c) 2014-2020, The Regents of the University of
  * California, through Lawrence Berkeley National Laboratory (subject
  * to receipt of any required approvals from the U.S. Dept. of
  * Energy).  All rights reserved.
@@ -54,6 +54,8 @@ extern const char report_reverse[] ;
 extern const char report_accepted[] ;
 extern const char report_cookie[] ;
 extern const char report_connected[] ;
+extern const char report_authentication_succeeded[] ;
+extern const char report_authentication_failed[] ;
 extern const char report_window[] ;
 extern const char report_autotune[] ;
 extern const char report_omit_done[] ;
@@ -62,10 +64,15 @@ extern const char report_done[] ;
 extern const char report_read_lengths[] ;
 extern const char report_read_length_times[] ;
 extern const char report_bw_header[] ;
+extern const char report_bw_header_bidir[] ;
 extern const char report_bw_retrans_header[] ;
+extern const char report_bw_retrans_header_bidir[] ;
 extern const char report_bw_retrans_cwnd_header[] ;
+extern const char report_bw_retrans_cwnd_header_bidir[] ;
 extern const char report_bw_udp_header[] ;
+extern const char report_bw_udp_header_bidir[] ;
 extern const char report_bw_udp_sender_header[] ;
+extern const char report_bw_udp_sender_header_bidir[] ;
 extern const char report_bw_format[] ;
 extern const char report_bw_retrans_format[] ;
 extern const char report_bw_retrans_cwnd_format[] ;
@@ -93,6 +100,10 @@ extern const char report_local[] ;
 extern const char report_remote[] ;
 extern const char report_sender[] ;
 extern const char report_receiver[] ;
+extern const char report_sender_not_available_format[];
+extern const char report_sender_not_available_summary_format[];
+extern const char report_receiver_not_available_format[];
+extern const char report_receiver_not_available_summary_format[];
 
 extern const char report_tcpInfo[] ;
 extern const char report_tcpInfo[] ;
